/**
 *    Copyright (C) 2014 MongoDB Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the GNU Affero General Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#include "mongo/db/service_context.h"

#include "mongo/platform/atomic_word.h"

namespace mongo {

class ServiceContextNoop final : public ServiceContext {
public:
    StorageEngine* getGlobalStorageEngine() override;

    void initializeGlobalStorageEngine() override;

    void shutdownGlobalStorageEngineCleanly() override;

    void registerStorageEngine(const std::string& name,
                               const StorageEngine::Factory* factory) override;

    bool isRegisteredStorageEngine(const std::string& name) override;

    StorageFactoriesIterator* makeStorageFactoriesIterator() override;

    void killOperation(OperationContext* txn,
                       ErrorCodes::Error killCode = ErrorCodes::Interrupted) override;

    void killAllUserOperations(const OperationContext* txn, ErrorCodes::Error killCode) override;

    void setKillAllOperations() override;

    void unsetKillAllOperations() override;

    bool getKillAllOperations() override;

    void registerKillOpListener(KillOpListenerInterface* listener) override;

    std::unique_ptr<OperationContext> _newOpCtx(Client* client) override;

    void setOpObserver(std::unique_ptr<OpObserver> opObserver) override;

    OpObserver* getOpObserver() override;

private:
    AtomicUInt32 _nextOpId{1};
};

}  // namespace mongo
